/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package org.xlsx4j.sml;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.zip.CRC32;
import javax.xml.bind.JAXBContext;
import org.docx4j.XmlUtils;


public class JAXBListHelper<T> {

    final JAXBContext                   jc;
    final private List<T>               values;
    final private List<Integer>         hashedValues = new ArrayList<Integer>();

    public JAXBListHelper(List<T> t, JAXBContext jc) {
        this.values = t;
        this.jc = jc;
    }

    private static int getHash(byte[] data) {
        final CRC32 crc32 = new CRC32();
        crc32.update(data);
        return (int)crc32.getValue() | 1;
    }

    public int getOrApply(T t) {

        final byte[] sourceData = XmlUtils.marshalToByteArray(t, true, jc);
        final int sourceHash = getHash(sourceData);

        int i;
        for(i = 0; i<values.size();i++) {

            byte[] destData = null;
            T dest = values.get(i);

            // creating hash
            if(i>=hashedValues.size()) {
                destData = XmlUtils.marshalToByteArray(dest, true, jc);
                hashedValues.add(getHash(destData));
            }
            if(sourceHash==hashedValues.get(i)) {
                // hash found... now check for identity
                if(destData==null) {
                    destData = XmlUtils.marshalToByteArray(dest, true, jc);
                }
                if(Arrays.equals(sourceData, destData)) {
                    return i;
                }
            }
        }

        // adding new object
        hashedValues.add(sourceHash);
        values.add(t);
        return i;
    }
}
